@@ -54,12 +54,10 @@ class BaseApi(object):
             for k in coll:
                 if name and k.name == name:
                     return k
-                elif id and v.obj_id == id:
+                elif id and k.obj_id == id:
                     return k
 
-        if require and name is not None:
-            raise NotFound()
-        if require and id is not None:
+        if require:
             raise NotFound()
 
         return None
