@@ -102,3 +102,6 @@ class Clips(CollectionApi):
 
     def _short_details(self, obj):
         return dict(name=obj.name, scene=obj.scene.name, track=obj.track.name, obj_id=obj.obj_id)
+
+    def delete(self, id=None):
+        raise Exception("NOT IMPLEMENTED YET")
\ No newline at end of file
