@@ -24,4 +24,14 @@ class Instruments(CollectionApi):
 
     def edit(self, name=None, id:str=None, new_name:str=None, channel:int=None, device:str=None, min_octave:int=None, max_octave:int=None, base_octave:int=None, muted:bool=None, default_velocity:int=0):
         device = self.api.devices.lookup(device, require=True)
-        return self._generic_edit(name, locals())
\ No newline at end of file
+        return self._generic_edit(name, locals())
+
+    def delete(self, id=None):
+        obj = self.lookup(id=id, require=True)
+
+        for track in self.song.tracks:
+            if track.instruments:
+                track.instruments = [ x for x in track.instruments if x != obj ]
+
+        return self._generic_remove(obj.name)
+
