@@ -28,4 +28,7 @@ class Patterns(CollectionApi):
         if scale:
             scale = self.api.scales.lookup(scale, require=True)
         params = locals()
-        return self._generic_edit(name, params)
\ No newline at end of file
+        return self._generic_edit(name, params)
+
+    def delete(self, id=None):
+        raise Exception("NOT IMPLEMENTED YET")
\ No newline at end of file
