@@ -60,4 +60,7 @@ class Scales(CollectionApi):
         return self._generic_edit(name, params)
 
     def scale_types(self):
-        return scale_types()
\ No newline at end of file
+        return scale_types()
+
+    def delete(self, id=None):
+        raise Exception("NOT IMPLEMENTED YET")
\ No newline at end of file
