@@ -24,4 +24,7 @@ class Transforms(CollectionApi):
 
     def edit(self, name, id:str=None, new_name:str=None, slots:list=None, divide:int=1, applies_to:str=None, direction=None, auto_reset=None):
         params = locals()
-        return self._generic_edit(name, params)
\ No newline at end of file
+        return self._generic_edit(name, params)
+
+    def delete(self, id=None):
+        raise Exception("NOT IMPLEMENTED YET")
\ No newline at end of file
