@@ -339,7 +339,6 @@ class Clip(NewReferenceObject):
         player = Player(
             clip=self,
             song=song,
-            engine=engine_class(song=song, track=self.track, clip=self),
+            engine=engine_class(),
         )
-        player.engine.player = player
         return player
